/*************************************************/
/*                                               */
/*      mpi.h                                    */
/*      message passing interface in fork95      */
/*      v0.13                                    */
/*      13.09.1999 Christoph W. Kessler,         */
/*      Oliver Fritzen                           */
/*************************************************/

#include <fork.h>

/* Results of the compare operations */
/* These should stay ordered */

#define MPI_IDENT     0
#define MPI_CONGRUENT 1 /* ??? */
#define MPI_SIMILAR   2
#define MPI_UNEQUAL   3

/* Data types */

typedef int MPI_Datatype;

#define MPI_CHAR           ((MPI_Datatype)1)
#define MPI_UNSIGNED_CHAR  ((MPI_Datatype)2)
#define MPI_BYTE           ((MPI_Datatype)3)
#define MPI_SHORT          ((MPI_Datatype)4)
#define MPI_UNSIGNED_SHORT ((MPI_Datatype)5)
#define MPI_INT            ((MPI_Datatype)6)
#define MPI_UNSIGNED       ((MPI_Datatype)7)
#define MPI_LONG           ((MPI_Datatype)8)
#define MPI_UNSIGNED_LONG  ((MPI_Datatype)9)
#define MPI_FLOAT          ((MPI_Datatype)10)
#define MPI_DOUBLE         ((MPI_Datatype)11)
#define MPI_LONG_DOUBLE    ((MPI_Datatype)12)
#define MPI_LONG_LONG_INT  ((MPI_Datatype)13)

#define MPI_PACKED         ((MPI_Datatype)14)
#define MPI_LB             ((MPI_Datatype)15)
#define MPI_UB             ((MPI_Datatype)16)

#define MPI_FLOAT_INT      ((MPI_Datatype)17)
#define MPI_DOUBLE_INT     ((MPI_Datatype)18)
#define MPI_LONG_INT       ((MPI_Datatype)19)
#define MPI_SHORT_INT      ((MPI_Datatype)20)
#define MPI_2INT           ((MPI_Datatype)21)
#define MPI_LONG_DOUBLE_INT ((MPI_Datatype)22)
/* 
   The layouts for the types MPI_DOUBLE_INT etc are simply
   struct { 
   double var;
   int    loc;
   }
   */




#define SYSTEM_WORD_LENGTH 32
/* SYSTEM_WORD_LENGTH is the length of a word in bits */
/* (and, since SB-PRAM-words are 4 Byte SWL is 32 */



/* Groups */

typedef struct {
  int size;
  int *members;
  int *ranks;
} MPI_Group;

/* Communicators */

typedef struct {
  MPI_Group group;
} MPI_Comm;

/* parallel operations */

typedef int MPI_Op; 
#define MPI_SUM 1;
#define MPI_MULT 2;

/* MAX values */
#define MPI_MAX_PROCESSOR_NAME 256
#define MPI_MAX_ERROR_STRING   512
#define MPI_MAX_NAME_STRING     63      /* How long a name do you need ? */

/* Pre-defined constants */

#define MAILBOX_SIZE 100
#define MPI_UNDEFINED      (-32766)
#define MPI_UNDEFINED_RANK MPI_UNDEFINED
#define MPI_KEYVAL_INVALID 0

/* source wildcard alias */

#define MPI_ANY_SOURCE -1


/* tag wildcard alias */
#define MPI_ANY_TAG -1

/* 
 * Status object.  It is the only user-visible MPI data-structure 
 */

typedef struct { 
  int count;
  int MPI_SOURCE;
  int MPI_TAG;
  int MPI_ERROR;
  int private_count;
} MPI_Status;

extern pr MPI_Comm MPI_COMM_WORLD;
extern pr MPI_Group MPI_GROUP_EMPTY;

/* MPI functions (at least some of them) */

/* init and finalize */
extern int MPI_Init(int *, char ***);
extern int MPI_Finalize(void);

/* (some) communication functions */
extern int MPI_Send(const void *, int, MPI_Datatype, int, int, MPI_Comm);
extern int MPI_Bcast(const void *, int, MPI_Datatype, int, MPI_Comm);
extern int MPI_Recv(const void *, int, MPI_Datatype, int, int, MPI_Comm, MPI_Status *);


/* Comm functions */
extern int MPI_Comm_group(MPI_Comm, MPI_Group *);
extern int MPI_Comm_create(MPI_Comm, MPI_Group, MPI_Comm *);
extern int MPI_Comm_free(MPI_Comm *);
extern int MPI_Comm_size(MPI_Comm, int *);
extern int MPI_Comm_rank(MPI_Comm, int *);
extern int MPI_Comm_dup(MPI_Comm, MPI_Comm *);
/* extern int MPI_Comm_rank(MPI_Comm, int, int, MPI_Comm*); */


/* Group functions */
extern int MPI_Group_union(MPI_Group, MPI_Group, MPI_Group *);
extern int MPI_Group_intersection(MPI_Group, MPI_Group, MPI_Group *);
extern int MPI_Group_difference(MPI_Group, MPI_Group, MPI_Group *);
extern int MPI_Group_incl(MPI_Group, int, int *, MPI_Group *);
extern int MPI_Group_excl(MPI_Group, int, int *, MPI_Group *);
extern int MPI_Group_size(MPI_Group, int *);
extern int MPI_Group_rank(MPI_Group, int *);
extern int MPI_Group_compare(MPI_Group, MPI_Group, int *);
extern int MPI_Group_free(MPI_Group *);


/* other functions */

extern int MPI_Reduce(void *, void *, int, MPI_Datatype, 
		      MPI_Op, int, MPI_Comm);
int MPI_Sizeof(MPI_Datatype);
void print_group(MPI_Group group);


// CWK:
#define MPI_Barrier() barrier
#define MPI_Comm_Rank(com,ptr) *ptr=__PROC_NR__
#define MPI_Comm_Size(com,ptr) *ptr=__STARTED_PROCS__


