/** testSkel.c
 *
 *  generic functions for dataparallel skeletons 
 *  map, reduce, prefix
 *
 *  C. Kessler 990604
 */
#include <fork.h>   // always required in Fork programs
#include <io.h>     // printf() prototype 
#include <string.h> // memcpy() prototype
#include <assert.h>
#include <Matrix.h>
#include <skeletons.h>


// example application for floats:

straight void fadd( void *c, void *a, void *b ) 
 { *(float *)c = *(float *)a + *(float *)b; }

straight void fsqr( void *c, void *a ) 
 { *(float *)c = *(float *)a * *(float *)a; }

straight void fmul( void *c, void *a, void *b ) 
 { *(float *)c = *(float *)a * *(float *)b; }
 
sync void fdot( sh float *s, sh float *x, sh float *y, sh int n )
{
 sh float *tmp = (float *)shalloc( n * sizeof(float) );
 int i;
 farm pprintf("fdot(%d): %x=(%x,%x)\n", n, s, x, y );
 
 map2( fmul, (void **) tmp, (void **)x, (void **)y, n, sizeof(float) );
 //seq for(i=0;i<n;i++) *s += tmp[i]; // ((float *)x)[i] * ((float *)y)[i];
 reduce( fadd, s, (void **) tmp, n, sizeof(float) );
 shallfree();
}

int main(void)
{
  start {
    sh float s, *x, *c;
    sh int n = 5, m = 7;
    sh Matrix A;
    int i, j;
   
    x = (float *)shalloc( m * sizeof(float) );
    c = (float *)shalloc( n * sizeof(float) );
    farm
       forall( i, 0, m, __STARTED_PROCS__) 
          x[i] = (float) i;     // some initialization
    seq for(i=0; i<m; i++) printf("x[%d] = %f\n", i, x[i] );
    A = new_Matrix( n, m, 1 );
    seq
       for (i=0; i<n; i++)
          for (j=0; j<m; j++)
             Set( A, i, j, (float)(i+j) );
    seq printMatrix( A );
    initTracing( 100000 );
    startTracing();
    Map2( (sync void(*)(sh void*,sh void*,sh void*,sh int))fdot, 
          (void **)c, (void **)(A->data), (void *)x,
          n, 1, m, 0, sizeof(float) );
    stopTracing();
    //prefix( fadd, (void **) c, (void **)x, n, sizeof(float));
    //seq for(i=0; i<n; i++) printf("prefix[%d] = %f\n", i, c[i] );
    //map( fsqr, (void **) c, (void **)x, n, sizeof(float) );
    seq for(i=0; i<n; i++) printf("c[%d] = %f\n", i, c[i] );
    seq printf(" %f\n", s );
    writeTraceFile("skeletons.trv", "nested parallelism: Map2(map;reduce)");
  }
  barrier;
}

